@@ -1,7 +1,8 @@
-from dataclasses import dataclass
+import contextlib
 from typing import Dict, List, Optional
 
 import appdirs
+import attr
 import desert
 import marshmallow
 import yaml
@@ -16,29 +17,38 @@ def get_path():
     return appdirs.user_config_dir("plotman") + "/plotman.yaml"
 
 
-def get_validated_configs():
-    """Return a validated instance of the PlotmanConfig dataclass with data from plotman.yaml
+def read_configuration_text(config_path):
+    try:
+        with open(config_path, "r") as file:
+            return file.read()
+    except FileNotFoundError as e:
+        raise ConfigurationException(
+            f"No 'plotman.yaml' file exists at expected location: '{config_path}'. To generate "
+            f"default config file, run: 'plotman config generate'"
+        ) from e
+
+
+def get_validated_configs(config_text, config_path):
+    """Return a validated instance of PlotmanConfig with data from plotman.yaml
 
     :raises ConfigurationException: Raised when plotman.yaml is either missing or malformed
     """
     schema = desert.schema(PlotmanConfig)
-    config_file_path = get_path()
+    config_objects = yaml.load(config_text, Loader=yaml.SafeLoader)
+
     try:
-        with open(config_file_path, "r") as file:
-            config_file = yaml.load(file, Loader=yaml.SafeLoader)
-            return schema.load(config_file)
-    except FileNotFoundError as e:
-        raise ConfigurationException(
-            f"No 'plotman.yaml' file exists at expected location: '{config_file_path}'. To generate "
-            f"default config file, run: 'plotman config generate'"
-        ) from e
+        loaded = schema.load(config_objects)
     except marshmallow.exceptions.ValidationError as e:
-        raise ConfigurationException(f"Config file at: '{config_file_path}' is malformed") from e
+        raise ConfigurationException(
+            f"Config file at: '{config_path}' is malformed"
+        ) from e
+
+    return loaded
 
 
 # Data models used to deserializing/formatting plotman.yaml files.
 
-@dataclass
+@attr.frozen
 class Archive:
     rsyncd_module: str
     rsyncd_path: str
@@ -47,11 +57,11 @@ class Archive:
     rsyncd_user: str
     index: int = 0  # If not explicit, "index" will default to 0
 
-@dataclass
+@attr.frozen
 class TmpOverrides:
     tmpdir_max_jobs: Optional[int] = None
 
-@dataclass
+@attr.frozen
 class Directories:
     log: str
     tmp: List[str]
@@ -60,7 +70,7 @@ class Directories:
     tmp_overrides: Optional[Dict[str, TmpOverrides]] = None
     archive: Optional[Archive] = None
 
-@dataclass
+@attr.frozen
 class Scheduling:
     global_max_jobs: int
     global_stagger_m: int
@@ -70,7 +80,7 @@ class Scheduling:
     tmpdir_stagger_phase_minor: int
     tmpdir_stagger_phase_limit: int = 1  # If not explicit, "tmpdir_stagger_phase_limit" will default to 1
 
-@dataclass
+@attr.frozen
 class Plotting:
     k: int
     e: bool
@@ -80,13 +90,13 @@ class Plotting:
     farmer_pk: Optional[str] = None
     pool_pk: Optional[str] = None
 
-@dataclass
+@attr.frozen
 class UserInterface:
-    use_stty_size: bool
+    use_stty_size: bool = True
 
-@dataclass
+@attr.frozen
 class PlotmanConfig:
-    user_interface: UserInterface
     directories: Directories
     scheduling: Scheduling
     plotting: Plotting
+    user_interface: UserInterface = attr.ib(factory=UserInterface)
